import {
    epsilon, splitter, resulterrbound, estimate, vec,
    fast_expansion_sum_zeroelim, scale_expansion_zeroelim
} from './util.js';

const o3derrboundA = (7 + 56 * epsilon) * epsilon;
const o3derrboundB = (3 + 28 * epsilon) * epsilon;
const o3derrboundC = (26 + 288 * epsilon) * epsilon * epsilon;

const bc = vec(4);
const ca = vec(4);
const ab = vec(4);
const adet = vec(8);
const bdet = vec(8);
const cdet = vec(8);
const abdet = vec(16);
const fin1 = vec(192);
const fin2 = vec(192);
const at_b = vec(4);
const at_c = vec(4);
const bt_c = vec(4);
const bt_a = vec(4);
const ct_a = vec(4);
const ct_b = vec(4);
const bct = vec(8);
const cat = vec(8);
const abt = vec(8);
const u = vec(4);
const v = vec(12);
const w = vec(16);

function orient3dadapt(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz, permanent) {
    let finnow, finother, finswap, finlength;
    let adxtail, bdxtail, cdxtail;
    let adytail, bdytail, cdytail;
    let adztail, bdztail, cdztail;
    let at_blen, at_clen, bt_clen, bt_alen, ct_alen, ct_blen;
    let vlength, wlength;
    let negate;
    let bvirt, c, ahi, alo, bhi, blo, _i, _j, _k, _0, s1, s0, t1, t0, u3;

    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;
    const adz = az - dz;
    const bdz = bz - dz;
    const cdz = cz - dz;

    s1 = bdx * cdy;
    c = splitter * bdx;
    ahi = c - (c - bdx);
    alo = bdx - ahi;
    c = splitter * cdy;
    bhi = c - (c - cdy);
    blo = cdy - bhi;
    s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
    t1 = cdx * bdy;
    c = splitter * cdx;
    ahi = c - (c - cdx);
    alo = cdx - ahi;
    c = splitter * bdy;
    bhi = c - (c - bdy);
    blo = bdy - bhi;
    t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = s0 - t0;
    bvirt = s0 - _i;
    bc[0] = s0 - (_i + bvirt) + (bvirt - t0);
    _j = s1 + _i;
    bvirt = _j - s1;
    _0 = s1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - t1;
    bvirt = _0 - _i;
    bc[1] = _0 - (_i + bvirt) + (bvirt - t1);
    u3 = _j + _i;
    bvirt = u3 - _j;
    bc[2] = _j - (u3 - bvirt) + (_i - bvirt);
    bc[3] = u3;
    const alen = scale_expansion_zeroelim(4, bc, adz, adet);

    s1 = cdx * ady;
    c = splitter * cdx;
    ahi = c - (c - cdx);
    alo = cdx - ahi;
    c = splitter * ady;
    bhi = c - (c - ady);
    blo = ady - bhi;
    s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
    t1 = adx * cdy;
    c = splitter * adx;
    ahi = c - (c - adx);
    alo = adx - ahi;
    c = splitter * cdy;
    bhi = c - (c - cdy);
    blo = cdy - bhi;
    t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = s0 - t0;
    bvirt = s0 - _i;
    ca[0] = s0 - (_i + bvirt) + (bvirt - t0);
    _j = s1 + _i;
    bvirt = _j - s1;
    _0 = s1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - t1;
    bvirt = _0 - _i;
    ca[1] = _0 - (_i + bvirt) + (bvirt - t1);
    u3 = _j + _i;
    bvirt = u3 - _j;
    ca[2] = _j - (u3 - bvirt) + (_i - bvirt);
    ca[3] = u3;
    const blen = scale_expansion_zeroelim(4, ca, bdz, bdet);

    s1 = adx * bdy;
    c = splitter * adx;
    ahi = c - (c - adx);
    alo = adx - ahi;
    c = splitter * bdy;
    bhi = c - (c - bdy);
    blo = bdy - bhi;
    s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
    t1 = bdx * ady;
    c = splitter * bdx;
    ahi = c - (c - bdx);
    alo = bdx - ahi;
    c = splitter * ady;
    bhi = c - (c - ady);
    blo = ady - bhi;
    t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = s0 - t0;
    bvirt = s0 - _i;
    ab[0] = s0 - (_i + bvirt) + (bvirt - t0);
    _j = s1 + _i;
    bvirt = _j - s1;
    _0 = s1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - t1;
    bvirt = _0 - _i;
    ab[1] = _0 - (_i + bvirt) + (bvirt - t1);
    u3 = _j + _i;
    bvirt = u3 - _j;
    ab[2] = _j - (u3 - bvirt) + (_i - bvirt);
    ab[3] = u3;
    const clen = scale_expansion_zeroelim(4, ab, cdz, cdet);

    const ablen = fast_expansion_sum_zeroelim(alen, adet, blen, bdet, abdet);
    finlength = fast_expansion_sum_zeroelim(ablen, abdet, clen, cdet, fin1);

    let det = estimate(finlength, fin1);
    let errbound = o3derrboundB * permanent;
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    bvirt = ax - adx;
    adxtail = ax - (adx + bvirt) + (bvirt - dx);
    bvirt = bx - bdx;
    bdxtail = bx - (bdx + bvirt) + (bvirt - dx);
    bvirt = cx - cdx;
    cdxtail = cx - (cdx + bvirt) + (bvirt - dx);
    bvirt = ay - ady;
    adytail = ay - (ady + bvirt) + (bvirt - dy);
    bvirt = by - bdy;
    bdytail = by - (bdy + bvirt) + (bvirt - dy);
    bvirt = cy - cdy;
    cdytail = cy - (cdy + bvirt) + (bvirt - dy);
    bvirt = az - adz;
    adztail = az - (adz + bvirt) + (bvirt - dz);
    bvirt = bz - bdz;
    bdztail = bz - (bdz + bvirt) + (bvirt - dz);
    bvirt = cz - cdz;
    cdztail = cz - (cdz + bvirt) + (bvirt - dz);

    if (adxtail === 0 && bdxtail === 0 && cdxtail === 0 &&
        adytail === 0 && bdytail === 0 && cdytail === 0 &&
        adztail === 0 && bdztail === 0 && cdztail === 0) {
        return det;
    }

    errbound = o3derrboundC * permanent + resulterrbound * Math.abs(det);
    det +=
        adz * (bdx * cdytail + cdy * bdxtail - (bdy * cdxtail + cdx * bdytail)) + adztail * (bdx * cdy - bdy * cdx) +
        bdz * (cdx * adytail + ady * cdxtail - (cdy * adxtail + adx * cdytail)) + bdztail * (cdx * ady - cdy * adx) +
        cdz * (adx * bdytail + bdy * adxtail - (ady * bdxtail + bdx * adytail)) + cdztail * (adx * bdy - ady * bdx);
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    finnow = fin1;
    finother = fin2;

    if (adxtail === 0) {
        if (adytail === 0) {
            at_b[0] = 0;
            at_blen = 1;
            at_c[0] = 0;
            at_clen = 1;
        } else {
            negate = -adytail;
            s1 = negate * bdx;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            at_b[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            at_b[1] = s1;
            at_blen = 2;
            s1 = adytail * cdx;
            c = splitter * adytail;
            ahi = c - (c - adytail);
            alo = adytail - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            at_c[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            at_c[1] = s1;
            at_clen = 2;
        }
    } else {
        if (adytail === 0) {
            s1 = adxtail * bdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            at_b[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            at_b[1] = s1;
            at_blen = 2;
            negate = -adxtail;
            s1 = negate * cdy;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            at_c[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            at_c[1] = s1;
            at_clen = 2;
        } else {
            s1 = adxtail * bdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            t1 = adytail * bdx;
            c = splitter * adytail;
            ahi = c - (c - adytail);
            alo = adytail - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = s0 - t0;
            bvirt = s0 - _i;
            at_b[0] = s0 - (_i + bvirt) + (bvirt - t0);
            _j = s1 + _i;
            bvirt = _j - s1;
            _0 = s1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - t1;
            bvirt = _0 - _i;
            at_b[1] = _0 - (_i + bvirt) + (bvirt - t1);
            u3 = _j + _i;
            bvirt = u3 - _j;
            at_b[2] = _j - (u3 - bvirt) + (_i - bvirt);
            at_b[3] = u3;
            at_blen = 4;
            s1 = adytail * cdx;
            c = splitter * adytail;
            ahi = c - (c - adytail);
            alo = adytail - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            t1 = adxtail * cdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = s0 - t0;
            bvirt = s0 - _i;
            at_c[0] = s0 - (_i + bvirt) + (bvirt - t0);
            _j = s1 + _i;
            bvirt = _j - s1;
            _0 = s1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - t1;
            bvirt = _0 - _i;
            at_c[1] = _0 - (_i + bvirt) + (bvirt - t1);
            u3 = _j + _i;
            bvirt = u3 - _j;
            at_c[2] = _j - (u3 - bvirt) + (_i - bvirt);
            at_c[3] = u3;
            at_clen = 4;
        }
    }
    if (bdxtail === 0) {
        if (bdytail === 0) {
            bt_c[0] = 0;
            bt_clen = 1;
            bt_a[0] = 0;
            bt_alen = 1;
        } else {
            negate = -bdytail;
            s1 = negate * cdx;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            bt_c[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            bt_c[1] = s1;
            bt_clen = 2;
            s1 = bdytail * adx;
            c = splitter * bdytail;
            ahi = c - (c - bdytail);
            alo = bdytail - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            bt_a[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            bt_a[1] = s1;
            bt_alen = 2;
        }
    } else {
        if (bdytail === 0) {
            s1 = bdxtail * cdy;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            bt_c[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            bt_c[1] = s1;
            bt_clen = 2;
            negate = -bdxtail;
            s1 = negate * ady;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            bt_a[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            bt_a[1] = s1;
            bt_alen = 2;
        } else {
            s1 = bdxtail * cdy;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            t1 = bdytail * cdx;
            c = splitter * bdytail;
            ahi = c - (c - bdytail);
            alo = bdytail - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = s0 - t0;
            bvirt = s0 - _i;
            bt_c[0] = s0 - (_i + bvirt) + (bvirt - t0);
            _j = s1 + _i;
            bvirt = _j - s1;
            _0 = s1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - t1;
            bvirt = _0 - _i;
            bt_c[1] = _0 - (_i + bvirt) + (bvirt - t1);
            u3 = _j + _i;
            bvirt = u3 - _j;
            bt_c[2] = _j - (u3 - bvirt) + (_i - bvirt);
            bt_c[3] = u3;
            bt_clen = 4;
            s1 = bdytail * adx;
            c = splitter * bdytail;
            ahi = c - (c - bdytail);
            alo = bdytail - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            t1 = bdxtail * ady;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = s0 - t0;
            bvirt = s0 - _i;
            bt_a[0] = s0 - (_i + bvirt) + (bvirt - t0);
            _j = s1 + _i;
            bvirt = _j - s1;
            _0 = s1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - t1;
            bvirt = _0 - _i;
            bt_a[1] = _0 - (_i + bvirt) + (bvirt - t1);
            u3 = _j + _i;
            bvirt = u3 - _j;
            bt_a[2] = _j - (u3 - bvirt) + (_i - bvirt);
            bt_a[3] = u3;
            bt_alen = 4;
        }
    }
    if (cdxtail === 0) {
        if (cdytail === 0) {
            ct_a[0] = 0;
            ct_alen = 1;
            ct_b[0] = 0;
            ct_blen = 1;
        } else {
            negate = -cdytail;
            s1 = negate * adx;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            ct_a[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            ct_a[1] = s1;
            ct_alen = 2;
            s1 = cdytail * bdx;
            c = splitter * cdytail;
            ahi = c - (c - cdytail);
            alo = cdytail - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            ct_b[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            ct_b[1] = s1;
            ct_blen = 2;
        }
    } else {
        if (cdytail === 0) {
            s1 = cdxtail * ady;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            ct_a[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            ct_a[1] = s1;
            ct_alen = 2;
            negate = -cdxtail;
            s1 = negate * bdy;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            ct_b[0] = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            ct_b[1] = s1;
            ct_blen = 2;
        } else {
            s1 = cdxtail * ady;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            t1 = cdytail * adx;
            c = splitter * cdytail;
            ahi = c - (c - cdytail);
            alo = cdytail - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = s0 - t0;
            bvirt = s0 - _i;
            ct_a[0] = s0 - (_i + bvirt) + (bvirt - t0);
            _j = s1 + _i;
            bvirt = _j - s1;
            _0 = s1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - t1;
            bvirt = _0 - _i;
            ct_a[1] = _0 - (_i + bvirt) + (bvirt - t1);
            u3 = _j + _i;
            bvirt = u3 - _j;
            ct_a[2] = _j - (u3 - bvirt) + (_i - bvirt);
            ct_a[3] = u3;
            ct_alen = 4;
            s1 = cdytail * bdx;
            c = splitter * cdytail;
            ahi = c - (c - cdytail);
            alo = cdytail - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            t1 = cdxtail * bdy;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = s0 - t0;
            bvirt = s0 - _i;
            ct_b[0] = s0 - (_i + bvirt) + (bvirt - t0);
            _j = s1 + _i;
            bvirt = _j - s1;
            _0 = s1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - t1;
            bvirt = _0 - _i;
            ct_b[1] = _0 - (_i + bvirt) + (bvirt - t1);
            u3 = _j + _i;
            bvirt = u3 - _j;
            ct_b[2] = _j - (u3 - bvirt) + (_i - bvirt);
            ct_b[3] = u3;
            ct_blen = 4;
        }
    }

    const bctlen = fast_expansion_sum_zeroelim(bt_clen, bt_c, ct_blen, ct_b, bct);
    wlength = scale_expansion_zeroelim(bctlen, bct, adz, w);
    finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
    finswap = finnow; finnow = finother; finother = finswap;

    const catlen = fast_expansion_sum_zeroelim(ct_alen, ct_a, at_clen, at_c, cat);
    wlength = scale_expansion_zeroelim(catlen, cat, bdz, w);
    finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
    finswap = finnow; finnow = finother; finother = finswap;

    const abtlen = fast_expansion_sum_zeroelim(at_blen, at_b, bt_alen, bt_a, abt);
    wlength = scale_expansion_zeroelim(abtlen, abt, cdz, w);
    finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
    finswap = finnow; finnow = finother; finother = finswap;

    if (adztail !== 0) {
        vlength = scale_expansion_zeroelim(4, bc, adztail, v);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, vlength, v, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (bdztail !== 0) {
        vlength = scale_expansion_zeroelim(4, ca, bdztail, v);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, vlength, v, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (cdztail !== 0) {
        vlength = scale_expansion_zeroelim(4, ab, cdztail, v);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, vlength, v, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }

    if (adxtail !== 0) {
        if (bdytail !== 0) {
            s1 = adxtail * bdytail;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdytail;
            bhi = c - (c - bdytail);
            blo = bdytail - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * cdz;
            bhi = c - (c - cdz);
            blo = cdz - bhi;
            _i = s0 * cdz;
            c = splitter * s0;
            ahi = c - (c - s0);
            alo = s0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = s1 * cdz;
            c = splitter * s1;
            ahi = c - (c - s1);
            alo = s1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (cdztail !== 0) {
                c = splitter * cdztail;
                bhi = c - (c - cdztail);
                blo = cdztail - bhi;
                _i = s0 * cdztail;
                c = splitter * s0;
                ahi = c - (c - s0);
                alo = s0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = s1 * cdztail;
                c = splitter * s1;
                ahi = c - (c - s1);
                alo = s1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
        if (cdytail !== 0) {
            negate = -adxtail;
            s1 = negate * cdytail;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * cdytail;
            bhi = c - (c - cdytail);
            blo = cdytail - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * bdz;
            bhi = c - (c - bdz);
            blo = bdz - bhi;
            _i = s0 * bdz;
            c = splitter * s0;
            ahi = c - (c - s0);
            alo = s0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = s1 * bdz;
            c = splitter * s1;
            ahi = c - (c - s1);
            alo = s1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (bdztail !== 0) {
                c = splitter * bdztail;
                bhi = c - (c - bdztail);
                blo = bdztail - bhi;
                _i = s0 * bdztail;
                c = splitter * s0;
                ahi = c - (c - s0);
                alo = s0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = s1 * bdztail;
                c = splitter * s1;
                ahi = c - (c - s1);
                alo = s1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
    }
    if (bdxtail !== 0) {
        if (cdytail !== 0) {
            s1 = bdxtail * cdytail;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdytail;
            bhi = c - (c - cdytail);
            blo = cdytail - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * adz;
            bhi = c - (c - adz);
            blo = adz - bhi;
            _i = s0 * adz;
            c = splitter * s0;
            ahi = c - (c - s0);
            alo = s0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = s1 * adz;
            c = splitter * s1;
            ahi = c - (c - s1);
            alo = s1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (adztail !== 0) {
                c = splitter * adztail;
                bhi = c - (c - adztail);
                blo = adztail - bhi;
                _i = s0 * adztail;
                c = splitter * s0;
                ahi = c - (c - s0);
                alo = s0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = s1 * adztail;
                c = splitter * s1;
                ahi = c - (c - s1);
                alo = s1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
        if (adytail !== 0) {
            negate = -bdxtail;
            s1 = negate * adytail;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * adytail;
            bhi = c - (c - adytail);
            blo = adytail - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * cdz;
            bhi = c - (c - cdz);
            blo = cdz - bhi;
            _i = s0 * cdz;
            c = splitter * s0;
            ahi = c - (c - s0);
            alo = s0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = s1 * cdz;
            c = splitter * s1;
            ahi = c - (c - s1);
            alo = s1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (cdztail !== 0) {
                c = splitter * cdztail;
                bhi = c - (c - cdztail);
                blo = cdztail - bhi;
                _i = s0 * cdztail;
                c = splitter * s0;
                ahi = c - (c - s0);
                alo = s0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = s1 * cdztail;
                c = splitter * s1;
                ahi = c - (c - s1);
                alo = s1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
    }
    if (cdxtail !== 0) {
        if (adytail !== 0) {
            s1 = cdxtail * adytail;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * adytail;
            bhi = c - (c - adytail);
            blo = adytail - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * bdz;
            bhi = c - (c - bdz);
            blo = bdz - bhi;
            _i = s0 * bdz;
            c = splitter * s0;
            ahi = c - (c - s0);
            alo = s0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = s1 * bdz;
            c = splitter * s1;
            ahi = c - (c - s1);
            alo = s1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (bdztail !== 0) {
                c = splitter * bdztail;
                bhi = c - (c - bdztail);
                blo = bdztail - bhi;
                _i = s0 * bdztail;
                c = splitter * s0;
                ahi = c - (c - s0);
                alo = s0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = s1 * bdztail;
                c = splitter * s1;
                ahi = c - (c - s1);
                alo = s1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
        if (bdytail !== 0) {
            negate = -cdxtail;
            s1 = negate * bdytail;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * bdytail;
            bhi = c - (c - bdytail);
            blo = bdytail - bhi;
            s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * adz;
            bhi = c - (c - adz);
            blo = adz - bhi;
            _i = s0 * adz;
            c = splitter * s0;
            ahi = c - (c - s0);
            alo = s0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = s1 * adz;
            c = splitter * s1;
            ahi = c - (c - s1);
            alo = s1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (adztail !== 0) {
                c = splitter * adztail;
                bhi = c - (c - adztail);
                blo = adztail - bhi;
                _i = s0 * adztail;
                c = splitter * s0;
                ahi = c - (c - s0);
                alo = s0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = s1 * adztail;
                c = splitter * s1;
                ahi = c - (c - s1);
                alo = s1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
    }

    if (adztail !== 0) {
        wlength = scale_expansion_zeroelim(bctlen, bct, adztail, w);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (bdztail !== 0) {
        wlength = scale_expansion_zeroelim(catlen, cat, bdztail, w);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (cdztail !== 0) {
        wlength = scale_expansion_zeroelim(abtlen, abt, cdztail, w);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }

    return finnow[finlength - 1];
}

export function orient3d(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz) {
    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;
    const adz = az - dz;
    const bdz = bz - dz;
    const cdz = cz - dz;

    const bdxcdy = bdx * cdy;
    const cdxbdy = cdx * bdy;

    const cdxady = cdx * ady;
    const adxcdy = adx * cdy;

    const adxbdy = adx * bdy;
    const bdxady = bdx * ady;

    const det =
        adz * (bdxcdy - cdxbdy) +
        bdz * (cdxady - adxcdy) +
        cdz * (adxbdy - bdxady);

    const permanent =
        (Math.abs(bdxcdy) + Math.abs(cdxbdy)) * Math.abs(adz) +
        (Math.abs(cdxady) + Math.abs(adxcdy)) * Math.abs(bdz) +
        (Math.abs(adxbdy) + Math.abs(bdxady)) * Math.abs(cdz);

    const errbound = o3derrboundA * permanent;
    if (det > errbound || -det > errbound) {
        return det;
    }

    return orient3dadapt(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz, permanent);
}

export function orient3dfast(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz) {
    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;
    const adz = az - dz;
    const bdz = bz - dz;
    const cdz = cz - dz;

    return adx * (bdy * cdz - bdz * cdy) +
        bdx * (cdy * adz - cdz * ady) +
        cdx * (ady * bdz - adz * bdy);
}
